@@ -2,6 +2,7 @@ mod img;
 
 use crate::persist::{Config, ExtractionConfig, Paths};
 use crate::theme::Theme;
+use color_eyre::eyre::{Result, WrapErr};
 use log::{error, info, trace, warn};
 use std::collections::hash_map::DefaultHasher;
 use std::fs::File;
@@ -27,30 +28,28 @@ enum Extractors {
 }
 
 pub trait Extractor {
-    type Err: std::error::Error;
-
-    fn extract(&self, config: &ExtractionConfig) -> Result<Theme, Self::Err>;
-    fn hash<H: Hasher>(&self, state: &mut H) -> Result<(), Self::Err>;
+    fn extract(&self, config: &ExtractionConfig) -> Result<Theme>;
+    fn hash<H: Hasher>(&self, state: &mut H) -> Result<()>;
 }
 
 // TODO: Rewrite with macros
 impl Extractors {
-    fn extract(&self, config: &ExtractionConfig) -> Result<Theme, std::io::Error> {
+    fn extract(&self, config: &ExtractionConfig) -> Result<Theme> {
         match self {
             Self::Image(img) => img.extract(config),
         }
     }
 
-    fn hash<H: Hasher>(&self, state: &mut H) -> Result<(), std::io::Error> {
+    fn hash<H: Hasher>(&self, state: &mut H) -> Result<()> {
         match self {
             Self::Image(img) => img.hash(state),
         }
     }
 
-    fn cache_path(&self, paths: &Paths) -> Result<PathBuf, std::io::Error> {
+    fn cache_path(&self, paths: &Paths) -> Result<PathBuf> {
         let hash = {
             let mut hasher = DefaultHasher::default();
-            self.hash(&mut hasher)?;
+            self.hash(&mut hasher).wrap_err("Failed to generate caching ID for extraction")?;
             hasher.finish()
         };
 
@@ -59,15 +58,16 @@ impl Extractors {
 }
 
 impl crate::Command for Opt {
-    type Err = std::io::Error;
-
-    fn run(&self, paths: &Paths, config: &Config) -> Result<Theme, Self::Err> {
-        let cache_path = self.extractor.cache_path(&paths)?;
+    fn run(&self, paths: &Paths, config: &Config) -> Result<Theme> {
+        let cache_path = self.extractor.cache_path(&paths).wrap_err("Failed to find extraction cache location")?;
 
         let theme = if self.cache && cache_path.exists() {
             info!("Cache hit; using cached theme...");
 
-            serde_json::from_reader(File::open(&cache_path)?)?
+            serde_json::from_reader(
+                File::open(&cache_path).wrap_err("Failed reading cached theme file")?,
+            )
+            .wrap_err("Failed deserializing cached theme file")?
         } else {
             if self.cache {
                 info!("Cache missed; extracting theme...");
@@ -75,17 +75,21 @@ impl crate::Command for Opt {
                 info!("Extracting theme...");
             }
 
-            self.extractor.extract(&config.extraction)?
+            self.extractor
+                .extract(&config.extraction)
+                .wrap_err("Failed to extract theme")?
         };
 
         trace!("Caching extracted theme...");
         File::create(&cache_path)
             .map(|file| {
                 serde_json::to_writer(file, &theme)
-                    .unwrap_or_else(|_| warn!("Failed to write theme to cache file."))
+                    .unwrap_or_else(|_| warn!("Failed to write theme to cache file"))
             })
-            .unwrap_or_else(|_| error!("Failed to create theme cache file."));
+            .unwrap_or_else(|_| error!("Failed to create theme cache file"));
 
         Ok(theme)
     }
 }
+
+
